searchData={"items":[{"type":"task","title":"mix rambla.rabbit.exchange","doc":"Mix task to deal with exchanges in the target RabbitMQ.\n\nThis is helpful to orchestrate target RabbitMQ when deploying\nto docker. Allows to create and delete the exchange.\n\nLoads the setting from `config :rambla, :amqp` if no connection\nis provided in parameters.","ref":"Mix.Tasks.Rambla.Rabbit.Exchange.html"},{"type":"task","title":"Command line options - mix rambla.rabbit.exchange","doc":"* -c - the connection string\n  * -o - the list of options without spaces, separated by comma","ref":"Mix.Tasks.Rambla.Rabbit.Exchange.html#module-command-line-options"},{"type":"task","title":"Options - mix rambla.rabbit.exchange","doc":"#","ref":"Mix.Tasks.Rambla.Rabbit.Exchange.html#module-options"},{"type":"task","title":"Options for `create` - mix rambla.rabbit.exchange","doc":"* **`type`** - One of four possible values below. Defaults to `:direct`.\n    * `direct`\n    * `fanout`\n    * `topic`\n    * `headers`\n  * `durable` - If set, keeps the Exchange between restarts of the broker;\n  * `auto_delete` - If set, deletes the Exchange once all queues\n    unbind from it;\n  * `passive` - If set, returns an error if the Exchange does not\n    already exist;\n  * `internal` - If set, the exchange may not be used directly by\n    publishers, but only when bound to other exchanges. Internal exchanges are used to construct wiring that is not visible to applications.\n  * `no_wait` - If set, the declare operation is asynchronous.\n    Defaults to false.\n  * `arguments` - A list of arguments to pass when declaring\n    (of type AMQP.arguments/0). See the README for more information. Defaults to [].\n\n#","ref":"Mix.Tasks.Rambla.Rabbit.Exchange.html#module-options-for-create"},{"type":"task","title":"Options for `delete` - mix rambla.rabbit.exchange","doc":"* `if_unused` - If set, the server will only delete the exchange\n    if it has no queue bindings.\n  * `no_wait` - If set, the delete operation is asynchronous.","ref":"Mix.Tasks.Rambla.Rabbit.Exchange.html#module-options-for-delete"},{"type":"type","title":"Mix.Tasks.Rambla.Rabbit.Exchange.command/0","doc":"","ref":"Mix.Tasks.Rambla.Rabbit.Exchange.html#t:command/0"},{"type":"task","title":"mix rambla.rabbit.queue","doc":"Mix task to deal with queues in the target RabbitMQ.\n\nThis is helpful to orchestrate target RabbitMQ when deploying\nto docker. Allows to create, delete, purge and query status of\nthe queue. Also, `bind` and `unbind` commands are supported,\nboth require `exchange:...` option to be passed.\n\nLoads the setting from `config :rambla, :amqp` if no connection\nis provided in parameters.","ref":"Mix.Tasks.Rambla.Rabbit.Queue.html"},{"type":"task","title":"Command line options - mix rambla.rabbit.queue","doc":"* -c - the connection string\n  * -o - the list of options without spaces, separated by comma","ref":"Mix.Tasks.Rambla.Rabbit.Queue.html#module-command-line-options"},{"type":"task","title":"Options - mix rambla.rabbit.queue","doc":"#","ref":"Mix.Tasks.Rambla.Rabbit.Queue.html#module-options"},{"type":"task","title":"Options for `create` - mix rambla.rabbit.queue","doc":"* `durable` - If set, keeps the Queue between restarts\n    of the broker. Defaults to false.\n  * `auto_delete` - If set, deletes the Queue once all\n    subscribers disconnect. Defaults to false.\n  * `exclusive` - If set, only one subscriber can consume\n    from the Queue. Defaults to false.\n  * `passive` - If set, raises an error unless the queue\n    already exists. Defaults to false.\n  * `no_wait` - If set, the declare operation is asynchronous.\n    Defaults to false.\n  * `arguments` - A list of arguments to pass when declaring\n    (of type AMQP.arguments/0). See the README for more information. Defaults to [].\n\n#","ref":"Mix.Tasks.Rambla.Rabbit.Queue.html#module-options-for-create"},{"type":"task","title":"Options for `delete` - mix rambla.rabbit.queue","doc":"* `if_unused` - If set, the server will only delete the queue\n    if it has no consumers. If the queue has consumers, it’s\n    not deleted and an error is returned.\n  * `if_empty` - If set, the server will only delete the queue\n    if it has no messages.\n  * `no_wait` - If set, the delete operation is asynchronous.","ref":"Mix.Tasks.Rambla.Rabbit.Queue.html#module-options-for-delete"},{"type":"type","title":"Mix.Tasks.Rambla.Rabbit.Queue.command/0","doc":"","ref":"Mix.Tasks.Rambla.Rabbit.Queue.html#t:command/0"},{"type":"module","title":"Rambla","doc":"Interface for the message publishing through `Rambla`.\n\n`Rambla` maintains connection pools with `Finitomata.Pool` for each service.\n\nThe typical config for `Rambla` service follows the pattern introduced by\n  `AMQP` library:\n\n```elixir\n‹service›: [\n  connections: [\n    ‹connection_name›: [‹key_1›: ‹value_1›, …]\n  ],\n  channels: [\n    ‹channel_name›: [\n      connection: ‹connection_name›,\n      options: [‹key_1›: ‹value_1›, …]\n    ]\n  ]\n]\n```\n\nTo start pools, simply embed `Rambla` into the supervision tree, it’d \n  start a supervisor with children for all the configured services.\n\nThe configuration of the service implies all the `Rambla`’s code for it will\n  be compiled, but the dependency itself must be added to the `deps` section\n  of the `Mix.Project` file.\n\nThe excerpt from the `Rambla.MixProject` itself follows\n\n```elixir\n  # optional backends\n  {:amqp, \"~> 3.0\", optional: true},\n  {:redix, \"~> 1.0\", optional: true},\n  {:gen_smtp, \"~> 0.4 or ~> 1.0\", optional: true},\n  {:telemetria, \"~> 0.4 or ~> 1.0\", optional: true},\n\n  # s3\n  {:ex_aws, \"~> 2.1\", optional: true},\n  {:ex_aws_s3, \"~> 2.0\", optional: true},\n  {:ex_aws_sts, \"~> 2.0\", optional: true},\n  {:hackney, \"~> 1.9\", optional: true},\n  {:sweet_xml, \"~> 0.6\", optional: true},\n  {:configparser_ex, \"~> 4.0\", optional: true},\n```\n\n---\n\nChannel names are used across connections to publish messages.\n  `Rambla.publish(:channel_1, message)` would publish the message to all channels\n  named `channel_1`.","ref":"Rambla.html"},{"type":"function","title":"Rambla.channels/0","doc":"Returns a map `%{‹service› => [‹channels›]}`","ref":"Rambla.html#channels/0"},{"type":"function","title":"Rambla.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Rambla.html#child_spec/1"},{"type":"function","title":"Rambla.pools/0","doc":"Returns the currently active pools","ref":"Rambla.html#pools/0"},{"type":"function","title":"Rambla.publish/3","doc":"Publishes the message to the target channels. The message structure depends on\nthe destination. For `RabbitMQ` is might be whatever, for `Smtp` it expects\nto have `to:`, `subject:` and `body:` fields.","ref":"Rambla.html#publish/3"},{"type":"function","title":"Rambla.publish_synch/2","doc":"Publishes the message to the destination synchronously, avoiding the pool.","ref":"Rambla.html#publish_synch/2"},{"type":"function","title":"Rambla.publish_synch/3","doc":"Publishes the message to the destination synchronously, avoiding the pool.\nUnlike `publish_synch/2`, allows to specify additional options per request.","ref":"Rambla.html#publish_synch/3"},{"type":"function","title":"Rambla.raw/2","doc":"Executes any arbitrary function in the context of one of workers in the\nrespective connection pool for the target.\n\nThe function would receive a pid of the connection process.","ref":"Rambla.html#raw/2"},{"type":"function","title":"Rambla.services/0","doc":"Returns a list of all the configured connections","ref":"Rambla.html#services/0"},{"type":"function","title":"Rambla.start_link/1","doc":"Starts the supervisor with all the configured services","ref":"Rambla.html#start_link/1"},{"type":"function","title":"Rambla.start_pools/0","doc":"Starts the pools configured in the `config.exs` / `releases.exs` file.\n\nThis call is equivalent to `start_pools(Application.get_env(:rambla, :pools))`.","ref":"Rambla.html#start_pools/0"},{"type":"function","title":"Rambla.start_pools/1","doc":"Starts the pools as specified by options (`map()` or `keyword()`)","ref":"Rambla.html#start_pools/1"},{"type":"behaviour","title":"Rambla.Handler","doc":"Default handler for AMQP connections.\n\n> ### `use Rambla.Handler` {: .info}\n>\n> When you `use Rambla.Handler`, the `Rambla.Handler` module will\n> do the following things for your module:\n>\n> - implement `@behaviour Finitomata.Pool.Actor` where `actor/2` will\n>   delegate to `handle_publish/2` expected to be implemented by this module,\n>   and overridable `on_result/2` and `on_error/2` will have a reasonable\n>   default implementation (debug for the former and warn and retry for the latter)\n> - set `@behaviour Rambla.Handler` to invite you to implement real publishing\n>   handler as `handle_publish/2`\n\n#","ref":"Rambla.Handler.html"},{"type":"behaviour","title":"Example - Rambla.Handler","doc":"```elixir\ndefmodule Rambla.Handler.Some do\n  use Rambla.Handler\n\n  @impl Rambla.Handler\n  def config do\n    [\n      connections: [\n        local_conn: [url: \"amqp://guest:guest@localhost:5672\"],\n      ],\n      channels: [\n        chan_1: [connection: :local_conn]\n      ]\n    ]\n  end\n\n  @impl Rambla.Handler\n  def handle_publish(payload, %{connection: conn} = state) do\n    SomeImpl.publish(conn, payload, state.options)\n  end \nend\n```","ref":"Rambla.Handler.html#module-example"},{"type":"callback","title":"Rambla.Handler.config/0","doc":"The callback to get to the configuration","ref":"Rambla.Handler.html#c:config/0"},{"type":"callback","title":"Rambla.Handler.external_servers/1","doc":"If specified, these services will be started before pools under `:rest_for_one`","ref":"Rambla.Handler.html#c:external_servers/1"},{"type":"callback","title":"Rambla.Handler.handle_publish/2","doc":"The callback to be implemented by the consumer of this code","ref":"Rambla.Handler.html#c:handle_publish/2"},{"type":"callback","title":"Rambla.Handler.on_fatal/2","doc":"The callback to be called when retries exhausted","ref":"Rambla.Handler.html#c:on_fatal/2"},{"type":"type","title":"Rambla.Handler.resolution/0","doc":"Callback resolution","ref":"Rambla.Handler.html#t:resolution/0"},{"type":"module","title":"Rambla.Handlers.Amqp","doc":"Default handler for _AMQP_ connections. For this handler to work properly,\n  one must include and start `:amqp` application with the config like\n\n```elixir\nconfig :amqp,\n  connections: [\n    local_conn: [url: \"amqp://guest:guest@localhost:5672\"],\n  ],\n  channels: [\n    chan_1: [connection: :local_conn]\n  ]\n\n# Then you can access the connection/channel via `Rambla.Handlers.Amqp` as\n\nRambla.Handlers.Amqp.publish(:chan_1, %{message: %{foo: 42}, exchange: \"rambla\"})\n```","ref":"Rambla.Handlers.Amqp.html"},{"type":"function","title":"Rambla.Handlers.Amqp.children_specs/1","doc":"The list of [`child_spec`](https://hexdocs.pm/elixir/Supervisor.html#t:child_spec/0) returned\n  to be embedded into a supervision tree.\n\nKnown options:\n\n- `connection_options` — a `keyword()` or a function of arity one, which is to receive\n  channel names and return connection options as a list\n- `count` — the number of workers in the pool\n- `child_opts` — the options to be passed to the worker’s spec (you won’t need those)\n\n#","ref":"Rambla.Handlers.Amqp.html#children_specs/1"},{"type":"function","title":"Example - Rambla.Handlers.Amqp.children_specs/1","doc":"```elixir\nRambla.Handlers.Redis.children_specs(\n  connection_options: [exchange: \"amq.direct\"], count: 3)\n```","ref":"Rambla.Handlers.Amqp.html#children_specs/1-example"},{"type":"function","title":"Rambla.Handlers.Amqp.extract_options/2","doc":"","ref":"Rambla.Handlers.Amqp.html#extract_options/2"},{"type":"function","title":"Rambla.Handlers.Amqp.publish/3","doc":"An interface to publish messages using the FSM pool.\n\nThe `id` is the specific to an implementation,\n  for `Amqp` it’d be the channel name, for instance.\n\nThe second parameter would be a payload, or, if the backend supports it,\n  the function of arity one, which would receive back the connection `pid`.\n\n#","ref":"Rambla.Handlers.Amqp.html#publish/3"},{"type":"function","title":"Example - Rambla.Handlers.Amqp.publish/3","doc":"```elixir\nRambla.Handlers.Amqp.publish :channel_name, %{foo: :bar}\n```","ref":"Rambla.Handlers.Amqp.html#publish/3-example"},{"type":"function","title":"Rambla.Handlers.Amqp.start_link/1","doc":"The entry point: this would start a supervisor with all the pools and stuff","ref":"Rambla.Handlers.Amqp.html#start_link/1"},{"type":"module","title":"Rambla.Handlers.Httpc","doc":"Default handler for _HTTP_ connections. For this handler to work properly,\n  one must configure it with \n\n```elixir\nconfig :rambla, :httpc,\n  connections: [\n    httpbin: \"https://httpbin.org/post\",\n    remote_conn: [scheme: \"https\", host: \"httpbin.org\", query: \"post\"]\n  ],\n  channels: [\n    chan_1: [connection: :httpbin, options: [headers: [{\"accept\", \"application/json\"}]]]\n  ]\n\n# Then you can access the connection/channel via `Rambla.Handlers.Amqp` as\n\nRambla.Handlers.Httpc.publish(:chan_1, %{message: %{foo: 42}, serializer: Jason})\n```","ref":"Rambla.Handlers.Httpc.html"},{"type":"function","title":"Rambla.Handlers.Httpc.children_specs/1","doc":"The list of [`child_spec`](https://hexdocs.pm/elixir/Supervisor.html#t:child_spec/0) returned\n  to be embedded into a supervision tree.\n\nKnown options:\n\n- `connection_options` — a `keyword()` or a function of arity one, which is to receive\n  channel names and return connection options as a list\n- `count` — the number of workers in the pool\n- `child_opts` — the options to be passed to the worker’s spec (you won’t need those)\n\n#","ref":"Rambla.Handlers.Httpc.html#children_specs/1"},{"type":"function","title":"Example - Rambla.Handlers.Httpc.children_specs/1","doc":"```elixir\nRambla.Handlers.Redis.children_specs(\n  connection_options: [exchange: \"amq.direct\"], count: 3)\n```","ref":"Rambla.Handlers.Httpc.html#children_specs/1-example"},{"type":"function","title":"Rambla.Handlers.Httpc.do_handle_publish/3","doc":"","ref":"Rambla.Handlers.Httpc.html#do_handle_publish/3"},{"type":"function","title":"Rambla.Handlers.Httpc.extract_options/2","doc":"","ref":"Rambla.Handlers.Httpc.html#extract_options/2"},{"type":"function","title":"Rambla.Handlers.Httpc.publish/3","doc":"An interface to publish messages using the FSM pool.\n\nThe `id` is the specific to an implementation,\n  for `Amqp` it’d be the channel name, for instance.\n\nThe second parameter would be a payload, or, if the backend supports it,\n  the function of arity one, which would receive back the connection `pid`.\n\n#","ref":"Rambla.Handlers.Httpc.html#publish/3"},{"type":"function","title":"Example - Rambla.Handlers.Httpc.publish/3","doc":"```elixir\nRambla.Handlers.Amqp.publish :channel_name, %{foo: :bar}\n```","ref":"Rambla.Handlers.Httpc.html#publish/3-example"},{"type":"function","title":"Rambla.Handlers.Httpc.start_link/1","doc":"The entry point: this would start a supervisor with all the pools and stuff","ref":"Rambla.Handlers.Httpc.html#start_link/1"},{"type":"behaviour","title":"Rambla.Handlers.Mock","doc":"Default handler for _Mock_ testing doubles. \n\nNormally, the `test.exs` config would have included all the channels\n  under `config :rambla, mock: […]` key. This would allow testing actual\n  interactions using `Mox` library.\n\nBy default it’d be simply send the message back to the caller.\n\n```elixir\nconfig :rambla, mock: [\n  connections: [mocked: :conn],\n  channels: [chan_0: [connection: :mocked]]\n]\n\n# Then you can access the connection/channel via `Rambla.Handlers.Amqp` as\n\nRambla.Handlers.Mock.publish(:chan_1, %{message: %{foo: 42}, serializer: Jason})\n```","ref":"Rambla.Handlers.Mock.html"},{"type":"function","title":"Rambla.Handlers.Mock.children_specs/1","doc":"The list of [`child_spec`](https://hexdocs.pm/elixir/Supervisor.html#t:child_spec/0) returned\n  to be embedded into a supervision tree.\n\nKnown options:\n\n- `connection_options` — a `keyword()` or a function of arity one, which is to receive\n  channel names and return connection options as a list\n- `count` — the number of workers in the pool\n- `child_opts` — the options to be passed to the worker’s spec (you won’t need those)\n\n#","ref":"Rambla.Handlers.Mock.html#children_specs/1"},{"type":"function","title":"Example - Rambla.Handlers.Mock.children_specs/1","doc":"```elixir\nRambla.Handlers.Redis.children_specs(\n  connection_options: [exchange: \"amq.direct\"], count: 3)\n```","ref":"Rambla.Handlers.Mock.html#children_specs/1-example"},{"type":"function","title":"Rambla.Handlers.Mock.do_handle_publish/4","doc":"","ref":"Rambla.Handlers.Mock.html#do_handle_publish/4"},{"type":"function","title":"Rambla.Handlers.Mock.extract_options/2","doc":"","ref":"Rambla.Handlers.Mock.html#extract_options/2"},{"type":"callback","title":"Rambla.Handlers.Mock.on_publish/3","doc":"","ref":"Rambla.Handlers.Mock.html#c:on_publish/3"},{"type":"function","title":"Rambla.Handlers.Mock.publish/3","doc":"An interface to publish messages using the FSM pool.\n\nThe `id` is the specific to an implementation,\n  for `Amqp` it’d be the channel name, for instance.\n\nThe second parameter would be a payload, or, if the backend supports it,\n  the function of arity one, which would receive back the connection `pid`.\n\n#","ref":"Rambla.Handlers.Mock.html#publish/3"},{"type":"function","title":"Example - Rambla.Handlers.Mock.publish/3","doc":"```elixir\nRambla.Handlers.Amqp.publish :channel_name, %{foo: :bar}\n```","ref":"Rambla.Handlers.Mock.html#publish/3-example"},{"type":"function","title":"Rambla.Handlers.Mock.start_link/1","doc":"The entry point: this would start a supervisor with all the pools and stuff","ref":"Rambla.Handlers.Mock.html#start_link/1"},{"type":"module","title":"Rambla.Handlers.Redis","doc":"Default handler for _Redis_ connections. For this handler to work properly,\n  one must configure it with \n\n```elixir\nconfig :rambla, :redis,\n  connections: [\n    local_conn: \"redis://localhost:6379/0\",\n    remote_conn: [host: \"example.com\", port: 6379, database: 0]\n  ],\n  channels: [\n    chan_1: [connection: :local_conn]\n  ]\n\n# Then you can access the connection/channel via `Rambla.Handlers.Amqp` as\n\nRambla.Handlers.Redis.publish(:chan_1, %{message: %{foo: 42}, serializer: Jason})\n```","ref":"Rambla.Handlers.Redis.html"},{"type":"function","title":"Rambla.Handlers.Redis.children_specs/1","doc":"The list of [`child_spec`](https://hexdocs.pm/elixir/Supervisor.html#t:child_spec/0) returned\n  to be embedded into a supervision tree.\n\nKnown options:\n\n- `connection_options` — a `keyword()` or a function of arity one, which is to receive\n  channel names and return connection options as a list\n- `count` — the number of workers in the pool\n- `child_opts` — the options to be passed to the worker’s spec (you won’t need those)\n\n#","ref":"Rambla.Handlers.Redis.html#children_specs/1"},{"type":"function","title":"Example - Rambla.Handlers.Redis.children_specs/1","doc":"```elixir\nRambla.Handlers.Redis.children_specs(\n  connection_options: [exchange: \"amq.direct\"], count: 3)\n```","ref":"Rambla.Handlers.Redis.html#children_specs/1-example"},{"type":"function","title":"Rambla.Handlers.Redis.extract_options/2","doc":"","ref":"Rambla.Handlers.Redis.html#extract_options/2"},{"type":"function","title":"Rambla.Handlers.Redis.publish/3","doc":"An interface to publish messages using the FSM pool.\n\nThe `id` is the specific to an implementation,\n  for `Amqp` it’d be the channel name, for instance.\n\nThe second parameter would be a payload, or, if the backend supports it,\n  the function of arity one, which would receive back the connection `pid`.\n\n#","ref":"Rambla.Handlers.Redis.html#publish/3"},{"type":"function","title":"Example - Rambla.Handlers.Redis.publish/3","doc":"```elixir\nRambla.Handlers.Amqp.publish :channel_name, %{foo: :bar}\n```","ref":"Rambla.Handlers.Redis.html#publish/3-example"},{"type":"function","title":"Rambla.Handlers.Redis.start_link/1","doc":"The entry point: this would start a supervisor with all the pools and stuff","ref":"Rambla.Handlers.Redis.html#start_link/1"},{"type":"module","title":"Rambla.Handlers.S3","doc":"Default handler for _S3 connections. For this handler to work properly,\n  one must configure it with \n\n```elixir\n# config :ex_aws,\n#   access_key_id: [{:system, \"AWS_ACCESS_KEY_ID\"}, {:awscli, \"default\", 30}, :instance_role],\n#   secret_access_key: [{:system, \"AWS_SECRET_ACCESS_KEY\"}, {:awscli, \"default\", 30}, :instance_role]\n\nconfig :ex_aws, \n  access_key_id: [{:system, \"AWS_ACCESS_KEY_ID\"}, :instance_role],\n  secret_access_key: [{:system, \"AWS_SECRET_ACCESS_KEY\"}, :instance_role],\n  hackney_opts: [follow_redirect: true, recv_timeout: 30_000],\n  region: {:system, \"AWS_REGION\"},\n  json_codec: Jason,\n  normalize_path: false,\n  retries: [\n    max_attempts: 1,\n    base_backoff_in_ms: 10,\n    max_backoff_in_ms: 10_000\n  ]\n\nconfig :rambla, :s3,\n  connections: [\n    bucket_1: [bucket: \"test-bucket\", path: \"\"]\n  ],\n  channels: [\n    chan_1: [\n      connection: :bucket_1,\n      options: [connector: ExAws]\n    ]\n  ]\n\n# Then you can access the connection/channel via `Rambla.Handlers.S3` as\n\nRambla.Handlers.S3.publish(:chan_1, %{message: \"Hi John!\", connector: Rambla.Mocks.ExAws})\n```","ref":"Rambla.Handlers.S3.html"},{"type":"function","title":"Rambla.Handlers.S3.children_specs/1","doc":"The list of [`child_spec`](https://hexdocs.pm/elixir/Supervisor.html#t:child_spec/0) returned\n  to be embedded into a supervision tree.\n\nKnown options:\n\n- `connection_options` — a `keyword()` or a function of arity one, which is to receive\n  channel names and return connection options as a list\n- `count` — the number of workers in the pool\n- `child_opts` — the options to be passed to the worker’s spec (you won’t need those)\n\n#","ref":"Rambla.Handlers.S3.html#children_specs/1"},{"type":"function","title":"Example - Rambla.Handlers.S3.children_specs/1","doc":"```elixir\nRambla.Handlers.Redis.children_specs(\n  connection_options: [exchange: \"amq.direct\"], count: 3)\n```","ref":"Rambla.Handlers.S3.html#children_specs/1-example"},{"type":"function","title":"Rambla.Handlers.S3.do_handle_publish/5","doc":"","ref":"Rambla.Handlers.S3.html#do_handle_publish/5"},{"type":"function","title":"Rambla.Handlers.S3.extract_options/2","doc":"","ref":"Rambla.Handlers.S3.html#extract_options/2"},{"type":"function","title":"Rambla.Handlers.S3.publish/3","doc":"An interface to publish messages using the FSM pool.\n\nThe `id` is the specific to an implementation,\n  for `Amqp` it’d be the channel name, for instance.\n\nThe second parameter would be a payload, or, if the backend supports it,\n  the function of arity one, which would receive back the connection `pid`.\n\n#","ref":"Rambla.Handlers.S3.html#publish/3"},{"type":"function","title":"Example - Rambla.Handlers.S3.publish/3","doc":"```elixir\nRambla.Handlers.Amqp.publish :channel_name, %{foo: :bar}\n```","ref":"Rambla.Handlers.S3.html#publish/3-example"},{"type":"function","title":"Rambla.Handlers.S3.start_link/1","doc":"The entry point: this would start a supervisor with all the pools and stuff","ref":"Rambla.Handlers.S3.html#start_link/1"},{"type":"module","title":"Rambla.Handlers.Smtp","doc":"Default handler for _SMTP_ connections. For this handler to work properly,\n  one must configure it with \n\n```elixir\nconfig :rambla, :smtp,\n  connections: [\n    gmail: [\n      # the smtp relay, e.g. \"smtp.gmail.com\"\n      relay: System.get_env(\"RAMBLA_SMTP_RELAY\", \"smtp.gmail.com\"),\n      # the username of the smtp relay e.g. \"me@gmail.com\"\n      username: System.get_env(\"RAMBLA_SMTP_USERNAME\"),\n      # the password of the smtp relay e.g. \"mypassword\"\n      password: System.get_env(\"RAMBLA_SMTP_PASSWORD\"),\n      # whether the smtp server needs authentication, valid values are if_available and always,\n      #   Defaults to if_available. If your smtp relay requires authentication set it to always\n      auth: :always,\n      # whether to connect on 465 in ssl mode, Defaults to false\n      ssl: true,\n      # valid values are always, never, if_available.\n      #   Most modern smtp relays use tls, so set this to always, Defaults to if_available\n      # tls: :always,\n      # used in ssl:connect, More info at http://erlang.org/doc/man/ssl.html ,\n      #   Defaults to [{versions , ['tlsv1', 'tlsv1.1', 'tlsv1.2']}],\n      #   This is merged with options listed at: https://github.com/gen-smtp/gen_smtp/blob/master/src/smtp_socket.erl#L46 .\n      #   Any options not present in this list will be ignored.\n      # tls_options: [versions: [:tlsv1, :\"tlsv1.1\", :\"tlsv1.2\"]],\n      # the hostname to be used by the smtp relay,\n      #   Defaults to: smtp_util:guess_FQDN().\n      #   The hostname on your computer might not be correct, so set this to a valid value.\n      hostname: System.get_env(\"RAMBLA_SMTP_HOSTNAME\", \"gmail.com\"),\n      from: %{\"Aleksei Matiushkin\" => \"matiouchkine@gmail.com\"}\n    ]\n  ],\n  channels: [\n    chan_3: [connection: :gmail]\n  ]\n\n# Then you can access the connection/channel via `Rambla.Handlers.Smtp` as\n\nRambla.Handlers.Smtp.publish(:chan_3, \"Hi John!\\nHow are you?\")\n```","ref":"Rambla.Handlers.Smtp.html"},{"type":"function","title":"Rambla.Handlers.Smtp.children_specs/1","doc":"The list of [`child_spec`](https://hexdocs.pm/elixir/Supervisor.html#t:child_spec/0) returned\n  to be embedded into a supervision tree.\n\nKnown options:\n\n- `connection_options` — a `keyword()` or a function of arity one, which is to receive\n  channel names and return connection options as a list\n- `count` — the number of workers in the pool\n- `child_opts` — the options to be passed to the worker’s spec (you won’t need those)\n\n#","ref":"Rambla.Handlers.Smtp.html#children_specs/1"},{"type":"function","title":"Example - Rambla.Handlers.Smtp.children_specs/1","doc":"```elixir\nRambla.Handlers.Redis.children_specs(\n  connection_options: [exchange: \"amq.direct\"], count: 3)\n```","ref":"Rambla.Handlers.Smtp.html#children_specs/1-example"},{"type":"function","title":"Rambla.Handlers.Smtp.do_handle_publish/3","doc":"","ref":"Rambla.Handlers.Smtp.html#do_handle_publish/3"},{"type":"function","title":"Rambla.Handlers.Smtp.extract_options/2","doc":"","ref":"Rambla.Handlers.Smtp.html#extract_options/2"},{"type":"function","title":"Rambla.Handlers.Smtp.publish/3","doc":"An interface to publish messages using the FSM pool.\n\nThe `id` is the specific to an implementation,\n  for `Amqp` it’d be the channel name, for instance.\n\nThe second parameter would be a payload, or, if the backend supports it,\n  the function of arity one, which would receive back the connection `pid`.\n\n#","ref":"Rambla.Handlers.Smtp.html#publish/3"},{"type":"function","title":"Example - Rambla.Handlers.Smtp.publish/3","doc":"```elixir\nRambla.Handlers.Amqp.publish :channel_name, %{foo: :bar}\n```","ref":"Rambla.Handlers.Smtp.html#publish/3-example"},{"type":"function","title":"Rambla.Handlers.Smtp.start_link/1","doc":"The entry point: this would start a supervisor with all the pools and stuff","ref":"Rambla.Handlers.Smtp.html#start_link/1"},{"type":"behaviour","title":"Rambla.Connection","doc":"The default behaviour for publishers. The common use case would be the module\nimplementing this behaviour opens a connection (and keep it opened,) and\npublishes messages as needed.\n\n---\n\n`Rambla.Connection` supports `options:` config that will be passed directly to\n`:poolboy` as [`Options`](https://github.com/devinus/poolboy#options).","ref":"Rambla.Connection.html"},{"type":"behaviour","title":"Example - Rambla.Connection","doc":"```elixir\nconfig :rambla,\n  amqp: […]\n  pools: [amqp: [options: [size: 2, max_overflow: 1], …]]\n```","ref":"Rambla.Connection.html#module-example"},{"type":"function","title":"Rambla.Connection.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Rambla.Connection.html#child_spec/1"},{"type":"callback","title":"Rambla.Connection.connect/1","doc":"Connects to the remote service and returns a connection object back","ref":"Rambla.Connection.html#c:connect/1"},{"type":"callback","title":"Rambla.Connection.publish/2","doc":"Publishes the message to the remote service using connection provided","ref":"Rambla.Connection.html#c:publish/2"},{"type":"function","title":"Rambla.Connection.start_link/1","doc":"Accepts options for the underlying connection (those will be passed to `connect/1`.)","ref":"Rambla.Connection.html#start_link/1"},{"type":"type","title":"Rambla.Connection.message/0","doc":"The accepted type of the message to be published","ref":"Rambla.Connection.html#t:message/0"},{"type":"type","title":"Rambla.Connection.messages/0","doc":"","ref":"Rambla.Connection.html#t:messages/0"},{"type":"type","title":"Rambla.Connection.outcome/0","doc":"The response type for the single request.\nContains a status and a response from remote service","ref":"Rambla.Connection.html#t:outcome/0"},{"type":"type","title":"Rambla.Connection.outcomes/0","doc":"The response type for the bulk request.\nContains a status and a response from remote service","ref":"Rambla.Connection.html#t:outcomes/0"},{"type":"type","title":"Rambla.Connection.t/0","doc":"The connection information","ref":"Rambla.Connection.html#t:t/0"},{"type":"module","title":"Rambla.Connection.Config","doc":"The connection settings as requested by connection provider","ref":"Rambla.Connection.Config.html"},{"type":"type","title":"Rambla.Connection.Config.t/0","doc":"The configuration of the real connection behind a pool","ref":"Rambla.Connection.Config.html#t:t/0"},{"type":"module","title":"Rambla.Amqp","doc":"Default connection implementation for 🐰 Rabbit.\n\n`publish/2` accepts the following options:\n\n- `exchange` [`binary()`, **mandatory**] the exchange to publish to\n- `queue` [`binary()`, **optional**] if passed, the queue will be created\n  and bound to the exchange; it’s slowing down publishing, but safer\n  for the cold RebbatMQ installation\n- `declare?`[`boolean()`, **optional**, _default_: `true`] if false\n  is passed, the exchange would not be declared; use it if the exchange\n  already surely exists to speed up the publishing\n- `routing_key` [`binary()`, **optional**, _default_: `\"\"`] if passed,\n  used as a routing key\n- `options` [`keyword()`, **optional**, _default_: `[]`] the options\n  to be passed as is to call to `AMQP.Basic.publish/5`\n\n---\n\nSince `v0.6.0` provides two `mix` tasks:\n\n- `mix rambla.rabbit.exchange` Operations with exchanges in RabbitMQ\n- `mix rambla.rabbit.queue`    Operations with queues in RabbitMQ\n\nTasks support arguments to be passed to RabbitMQ instance. Usage example:\n\n```\nmix rambla.rabbit.queue declare foo -o durable:true\n```","ref":"Rambla.Amqp.html"},{"type":"module","title":"Rambla.Http","doc":"Default connection implementation for 🕸️ HTTP.\n\nIt expects a message to be a map, containing the following fields:\n`:method`, `:path`, `:query`, `:body` _and_ the optional `:type`\nthat otherwise would be inferred from the body type.\n\nFor instance, this call would send a POST request with a JSON specified as body.\n\n```elixir\nRambla.publish(\n  Rambla.Http,\n  %{method: :post, body: %{message: \"I ❤ HTTP\"}}\n}\n```\n\nIf the second argument `message` is `binary()` it’s treated as an URL _and_\n`:get` is implied.\n\n---\n\nList of all possible options might be found in\n[`:httpc.request/4`](http://erlang.org/doc/man/httpc.html#request-4), names are preserved.","ref":"Rambla.Http.html"},{"type":"module","title":"Rambla.Process","doc":"Default connection implementation for 🕸️ Process message callback.\n\nFor instance, this call would send a message back to the calling process.\n\n```elixir\nRambla.publish(\n  Rambla.Process,\n  %{method: :post, body: %{message: \"I ❤ HTTP\"}}\n}\n```","ref":"Rambla.Process.html"},{"type":"module","title":"Rambla.Redis","doc":"Default connection implementation for 🔴 Redis.","ref":"Rambla.Redis.html"},{"type":"module","title":"Rambla.Smtp","doc":"Default connection implementation for 📧 SMTP.\n\nIt expects a message to be a map containing the following fields:\n`:to`, `:subject`, `:body` _and_ the optional `:from` that otherwise would be\ntaken from the global settings (`releases.mix`) from `[]:rambla, :pools, Rambla.Smtp]`.\n\nFor instance, this call would send an email to email:am@example.com with the\nrespective subject and body.\n\n```elixir\nRambla.publish(\n  Rambla.Smtp,\n  %{to: \"am@example.com\", subject: \"Hi there\", body: \"I ❤ SMTP\"}\n}\n```","ref":"Rambla.Smtp.html"},{"type":"module","title":"Rambla.Exception","doc":"Base type for all the `Rambla` exceptions.","ref":"Rambla.Exception.html"},{"type":"type","title":"Rambla.Exception.t/0","doc":"`Rambla` exception contains:\n\n- `reason` the string describing the error in human-readable form\n- `source` usually the type/action this error is originated from\n- `info` free-style map containig additional information (not displayed by default)","ref":"Rambla.Exception.html#t:t/0"},{"type":"extras","title":"Rambla","doc":"# Rambla    [![Kantox ❤ OSS](https://img.shields.io/badge/❤-kantox_oss-informational.svg)](https://kantox.com/)  ![Test](https://github.com/am-kantox/rambla/workflows/Test/badge.svg)  ![Dialyzer](https://github.com/am-kantox/rambla/workflows/Dialyzer/badge.svg)\n\n**Easy publishing to many different targets**","ref":"readme.html"},{"type":"extras","title":"Installation - Rambla","doc":"```elixir\ndef deps do\n  [\n    {:rambla, \"~> 0.4\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Supported backends - Rambla","doc":"- Rabbit (through [Amqp](https://hexdocs.pm/amqp/))\n- Redis (through [Redix](https://hexdocs.pm/redix))\n- Http (through [:httpc](http://erlang.org/doc/man/httpc.html))\n- Smtp (through [:gen_smtp](https://hexdocs.pm/gen_smtp))\n- S3 (through [:ex_aws_s3](https://hexdocs.pm/ex_aws_s3))\n- Slack (through [Envío](https://hexdocs.pm/envio)) [pending]","ref":"readme.html#supported-backends"},{"type":"extras","title":"Coming soon - Rambla","doc":"- AWS","ref":"readme.html#coming-soon"},{"type":"extras","title":"Changelog - Rambla","doc":"- **`1.1.1`** `Rambla.Handlers.Mock`\n- **`1.1.0`** Complete rewrite, all the services are now controlled by `Finitomata.Pool`\n- **`1.0.0`** Modern era update\n- **`0.16.3`** Better error reporting, logging of insuccessfuly HTTP requests\n- **`0.16.0`** Several instances of service + Mocks\n- **`0.15.0`** Hashring for selecting channels in AMQP backend within a single connection\n- **`0.14.4`** Accept headers for `:httpc` as map/keyword of binaries, `:gen_server_timeout` for `publish_synch/3`\n- **`0.14.0`** Use Tarearbol.Pool to manage channels behind AMQP connections\n- **`0.13.0`** Filter out connection params from logs\n- **`0.12.0`** `Rambla.publish_synch/3` to avoid pool while publishing\n- **`0.11.1`** Optional Boundary support for Telemetria\n- **`0.11.0`** Envío → Telemetria\n- **`0.9.3`** Envío broadcast to `:rambla` channel, with a type\n- **`0.9.0`** Divorce `Rambla` with `AMQP` and `Envio`\n- **`0.8.0`** `Rambla.raw/2` returning a worker from pool\n- **`0.6.5`** `RabbitMQ` → `bind`, `unbind`\n- **`0.6.3`** Auto-reenable tasks\n- **`0.6.2`** code cleanup, DRY\n- **`0.6.0`** `mix` tasks to deal with RabbitMQ\n- **`0.5.2`** graceful timeout, fix for optional `Envio` does not included\n- **`0.5.1`** performance fixes, do not require `queue` in call to Rabbit `publish/2`, `declare?: false` to not declare exchange every time\n- **`0.5.0`** bulk publisher\n- **`0.4.0`** `SMTP` publisher\n- **`0.3.0`** `HTTP` publisher","ref":"readme.html#changelog"},{"type":"extras","title":"Documentation - Rambla","doc":"- [https://hexdocs.pm/rambla](https://hexdocs.pm/rambla).","ref":"readme.html#documentation"},{"type":"extras","title":"Getting Started","doc":"# Getting Started","ref":"getting-started.html"},{"type":"extras","title":"Intro - Getting Started","doc":"`Rambla` provides the ability to publish messages to several different destinations. Destinations are supported via backends. The respective backend must be explicitly included into the list of extra applications of _target application_.\n\nFor each of the configured destinations, the pool of workers to talk to it is maintained. Each destination should be configured separately; the preferred way would be to use `config.exs` for local development and `releases.exs` for releases.","ref":"getting-started.html#intro"},{"type":"extras","title":"Configuration - Getting Started","doc":"The configuration is the keyword list with keys specifying backends and values—their initialization properties. Properties contain [_pool options_](https://github.com/devinus/poolboy/blob/master/README.md), _name_ and [_type_](https://hexdocs.pm/elixir/GenServer.html?#module-name-registration) of the worker, and _parameters_ to be passed as is to the underlying backend engine.","ref":"getting-started.html#configuration"},{"type":"extras","title":"Starting Pools - Getting Started","doc":"Upon target application start, [`Rambla.start_pools/0`](https://hexdocs.pm/rambla/Rambla.html#start_pools/0) function should be called to start pools. If providing a configuration through the file is not an option, it might be passed as is to the call to [`Rambla.start_pools/1`](https://hexdocs.pm/rambla/Rambla.html#start_pools/1). In this case, the static configuration from config files would be discarded.","ref":"getting-started.html#starting-pools"},{"type":"extras","title":"Configuration Example - Getting Started","doc":"Most parameters in the options have reasonable default values `:local` for `:type` and backend module name for worker name. Here is the full example, with all the backend on, and options set.\n\n```elixir\nconfig :rambla, :pools,\n  redis: [\n    params: [\n      host: System.get_env(\"REDIS_HOST\", \"127.0.0.1\"),\n      port: String.to_integer(System.get_env(\"REDIS_PORT\", \"6379\")),\n      password: System.get_env(\"REDIS_PASSWORD\", \"\"),\n      database: 0\n    ]\n  ],\n  amqp: [\n    options: [size: 5, max_overflow: 300],\n    params: [\n      host: System.get_env(\"RABBITMQ_HOST\", \"127.0.0.1\"),\n      port: String.to_integer(System.get_env(\"RABBITMQ_PORT\", \"5672\")),\n      username: System.get_env(\"RABBITMQ_USERNAME\", \"guest\"),\n      password: System.get_env(\"RABBITMQ_PASSWORD\", \"guest\")\n      virtual_host: System.get_env(\"RABBITMQ_VHOST\", \"/\"),\n      x_message_ttl: \"4000\"\n    ]\n  ],\n  http: [\n    options: [size: 25],\n    params: [\n      host: System.get_env(\"RAMBLA_HTTP_HOST\", \"127.0.0.1\"),\n      port: String.to_integer(System.get_env(\"RAMBLA_HTTP_PORT\", \"80\"))\n    ]\n  ],\n  smtp: [\n    options: [max_overflow: 10],\n    params: [\n      relay: System.get_env(\"RAMBLA_SMTP_RELAY\", \"smtp.gmail.com\"),\n      username: System.get_env(\"RAMBLA_SMTP_USERNAME\"),\n      password: System.get_env(\"RAMBLA_SMTP_PASSWORD\"),\n      auth: :always,\n      ssl: true,\n      hostname: System.get_env(\"RAMBLA_SMTP_HOSTNAME\"),\n      retries: 3,\n      from: %{\"Aleksei Matiushkin\" => \"aleksei@example.com\"}\n    ]\n  ]\n```","ref":"getting-started.html#configuration-example"},{"type":"extras","title":"Publishing - Getting Started","doc":"Publishing to the destination is as easy, as calling [`Rambla.html#publish/3`](https://hexdocs.pm/rambla/Rambla.html#publish/3) passing the destination (e. g. `Rambla.Amqp`,) message and optional configuration parameters.\n\nThe following would publish the message to previously configured `AMQP` connection:\n\n```elixir\nRambla.publish(Rambla.Amqp, %{foo: 42, bar: :baz}, exchange: \"barfoo\")\n```","ref":"getting-started.html#publishing"},{"type":"extras","title":"Testing Rambla","doc":"# Testing Rambla\n\nThere are two ways to test `Rambla` in the wild. One might do a granular testing as shown\nbelow with `S3` mock example.\n\n```elixir\nexpect(Rambla.Mocks.ExAws, :request, fn operation, %{} = _params ->\n  assert %ExAws.Operation.S3{} = operation\n  assert operation.http_method == :put\n  assert operation.bucket == \"test-bucket\"\n  assert operation.path == \"some/path\"\n\n  {:ok, %{body: \"file contents\"}}\nend)\n\nRambla.Handlers.S3.publish(:chan_1, %{message: \"file contents\"}, self())\nassert_receive {:transition, :success, _, _}, 1_000\n```\n\nAnother options would be to use `Rambla.Handlers.Mock` with a custom mock.\n\n```elixir\nexpect(Rambla.Mocks.Generic, :on_publish, fn name, message, %{} = _opts ->\n  assert name == :chan_0\n  assert message == \"file contents\"\n\n  {:ok, %{body: \"file contents\"}}\nend)\n\nRambla.publish(:chan_0, %{message: \"file contents\"}, self())\nassert_receive {:transition, :success, _, _}, 1_000\n```","ref":"testing.html"}],"content_type":"text/markdown"}